# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["CancelSubscription", "Data"]


class Data(BaseModel):
    id: str
    """Subscription ID"""


class CancelSubscription(BaseModel):
    """Response object"""

    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .addons import (
    AddonsResource,
    AsyncAddonsResource,
    AddonsResourceWithRawResponse,
    AsyncAddonsResourceWithRawResponse,
    AddonsResourceWithStreamingResponse,
    AsyncAddonsResourceWithStreamingResponse,
)
from .entitlements import (
    EntitlementsResource,
    AsyncEntitlementsResource,
    EntitlementsResourceWithRawResponse,
    AsyncEntitlementsResourceWithRawResponse,
    EntitlementsResourceWithStreamingResponse,
    AsyncEntitlementsResourceWithStreamingResponse,
)

__all__ = [
    "EntitlementsResource",
    "AsyncEntitlementsResource",
    "EntitlementsResourceWithRawResponse",
    "AsyncEntitlementsResourceWithRawResponse",
    "EntitlementsResourceWithStreamingResponse",
    "AsyncEntitlementsResourceWithStreamingResponse",
    "AddonsResource",
    "AsyncAddonsResource",
    "AddonsResourceWithRawResponse",
    "AsyncAddonsResourceWithRawResponse",
    "AddonsResourceWithStreamingResponse",
    "AsyncAddonsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["UsageReportParams", "Usage"]


class UsageReportParams(TypedDict, total=False):
    usages: Required[Iterable[Usage]]
    """A list of usage reports to be submitted in bulk"""


class Usage(TypedDict, total=False):
    """Single usage measurement"""

    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """Customer id"""

    feature_id: Required[Annotated[str, PropertyInfo(alias="featureId")]]
    """Feature id"""

    value: Required[int]
    """The value to report for usage"""

    created_at: Annotated[Union[str, datetime], PropertyInfo(alias="createdAt", format="iso8601")]
    """Timestamp of when the record was created"""

    dimensions: Dict[str, Union[str, float, bool]]
    """Additional dimensions for the usage report"""

    resource_id: Annotated[Optional[str], PropertyInfo(alias="resourceId")]
    """Resource id"""

    update_behavior: Annotated[Literal["DELTA", "SET"], PropertyInfo(alias="updateBehavior")]
    """The method by which the usage value should be updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["FeatureListFeaturesResponse", "EnumConfiguration", "UnitTransformation"]


class EnumConfiguration(BaseModel):
    display_name: str = FieldInfo(alias="displayName")
    """The display name for the enum configuration entity"""

    value: str
    """The unique value identifier for the enum configuration entity"""


class UnitTransformation(BaseModel):
    """Unit transformation to be applied to the reported usage"""

    divide: float
    """Divide usage by this number"""

    feature_units: Optional[str] = FieldInfo(alias="featureUnits", default=None)
    """Singular feature units after the transformation"""

    feature_units_plural: Optional[str] = FieldInfo(alias="featureUnitsPlural", default=None)
    """Plural feature units after the transformation"""

    round: Literal["UP", "DOWN"]
    """After division, either round the result up or down"""


class FeatureListFeaturesResponse(BaseModel):
    """Feature configuration object"""

    id: str
    """The unique identifier for the feature"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    description: Optional[str] = None
    """The description for the feature"""

    display_name: str = FieldInfo(alias="displayName")
    """The display name for the feature"""

    enum_configuration: Optional[List[EnumConfiguration]] = FieldInfo(alias="enumConfiguration", default=None)
    """The configuration data for the feature"""

    feature_status: Literal["NEW", "SUSPENDED", "ACTIVE"] = FieldInfo(alias="featureStatus")
    """The status of the feature"""

    feature_type: Literal["BOOLEAN", "NUMBER", "ENUM"] = FieldInfo(alias="featureType")
    """The type of the feature"""

    feature_units: Optional[str] = FieldInfo(alias="featureUnits", default=None)
    """The units for the feature"""

    feature_units_plural: Optional[str] = FieldInfo(alias="featureUnitsPlural", default=None)
    """The plural units for the feature"""

    metadata: Dict[str, str]
    """The additional metadata for the feature"""

    meter_type: Literal["None", "FLUCTUATING", "INCREMENTAL"] = FieldInfo(alias="meterType")
    """The meter type for the feature"""

    unit_transformation: Optional[UnitTransformation] = FieldInfo(alias="unitTransformation", default=None)
    """Unit transformation to be applied to the reported usage"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg.types.v1.addons import (
    AddonPackageEntitlement,
    EntitlementListResponse,
    EntitlementCreateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEntitlements:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Stigg) -> None:
        entitlement = client.v1.addons.entitlements.create(
            addon_id="addonId",
            entitlements=[
                {
                    "id": "id",
                    "type": "FEATURE",
                }
            ],
        )
        assert_matches_type(EntitlementCreateResponse, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Stigg) -> None:
        response = client.v1.addons.entitlements.with_raw_response.create(
            addon_id="addonId",
            entitlements=[
                {
                    "id": "id",
                    "type": "FEATURE",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = response.parse()
        assert_matches_type(EntitlementCreateResponse, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Stigg) -> None:
        with client.v1.addons.entitlements.with_streaming_response.create(
            addon_id="addonId",
            entitlements=[
                {
                    "id": "id",
                    "type": "FEATURE",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = response.parse()
            assert_matches_type(EntitlementCreateResponse, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            client.v1.addons.entitlements.with_raw_response.create(
                addon_id="",
                entitlements=[
                    {
                        "id": "id",
                        "type": "FEATURE",
                    }
                ],
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_overload_1(self, client: Stigg) -> None:
        entitlement = client.v1.addons.entitlements.update(
            id="id",
            addon_id="addonId",
            type="FEATURE",
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params_overload_1(self, client: Stigg) -> None:
        entitlement = client.v1.addons.entitlements.update(
            id="id",
            addon_id="addonId",
            type="FEATURE",
            behavior="Increment",
            description="description",
            display_name_override="displayNameOverride",
            enum_values=["string"],
            has_soft_limit=True,
            has_unlimited_usage=True,
            hidden_from_widgets=["PAYWALL"],
            is_custom=True,
            is_granted=True,
            monthly_reset_period_configuration={"according_to": "SubscriptionStart"},
            order=0,
            reset_period="YEAR",
            usage_limit=0,
            weekly_reset_period_configuration={"according_to": "SubscriptionStart"},
            yearly_reset_period_configuration={"according_to": "SubscriptionStart"},
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_overload_1(self, client: Stigg) -> None:
        response = client.v1.addons.entitlements.with_raw_response.update(
            id="id",
            addon_id="addonId",
            type="FEATURE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = response.parse()
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_overload_1(self, client: Stigg) -> None:
        with client.v1.addons.entitlements.with_streaming_response.update(
            id="id",
            addon_id="addonId",
            type="FEATURE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = response.parse()
            assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_overload_1(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            client.v1.addons.entitlements.with_raw_response.update(
                id="id",
                addon_id="",
                type="FEATURE",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.entitlements.with_raw_response.update(
                id="",
                addon_id="addonId",
                type="FEATURE",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_overload_2(self, client: Stigg) -> None:
        entitlement = client.v1.addons.entitlements.update(
            id="id",
            addon_id="addonId",
            type="CREDIT",
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params_overload_2(self, client: Stigg) -> None:
        entitlement = client.v1.addons.entitlements.update(
            id="id",
            addon_id="addonId",
            type="CREDIT",
            amount=1,
            behavior="Increment",
            cadence="MONTH",
            description="description",
            display_name_override="displayNameOverride",
            hidden_from_widgets=["PAYWALL"],
            is_custom=True,
            is_granted=True,
            order=0,
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_overload_2(self, client: Stigg) -> None:
        response = client.v1.addons.entitlements.with_raw_response.update(
            id="id",
            addon_id="addonId",
            type="CREDIT",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = response.parse()
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_overload_2(self, client: Stigg) -> None:
        with client.v1.addons.entitlements.with_streaming_response.update(
            id="id",
            addon_id="addonId",
            type="CREDIT",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = response.parse()
            assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_overload_2(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            client.v1.addons.entitlements.with_raw_response.update(
                id="id",
                addon_id="",
                type="CREDIT",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.entitlements.with_raw_response.update(
                id="",
                addon_id="addonId",
                type="CREDIT",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Stigg) -> None:
        entitlement = client.v1.addons.entitlements.list(
            "addonId",
        )
        assert_matches_type(EntitlementListResponse, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Stigg) -> None:
        response = client.v1.addons.entitlements.with_raw_response.list(
            "addonId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = response.parse()
        assert_matches_type(EntitlementListResponse, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Stigg) -> None:
        with client.v1.addons.entitlements.with_streaming_response.list(
            "addonId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = response.parse()
            assert_matches_type(EntitlementListResponse, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            client.v1.addons.entitlements.with_raw_response.list(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Stigg) -> None:
        entitlement = client.v1.addons.entitlements.delete(
            id="id",
            addon_id="addonId",
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Stigg) -> None:
        response = client.v1.addons.entitlements.with_raw_response.delete(
            id="id",
            addon_id="addonId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = response.parse()
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Stigg) -> None:
        with client.v1.addons.entitlements.with_streaming_response.delete(
            id="id",
            addon_id="addonId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = response.parse()
            assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            client.v1.addons.entitlements.with_raw_response.delete(
                id="id",
                addon_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.entitlements.with_raw_response.delete(
                id="",
                addon_id="addonId",
            )


class TestAsyncEntitlements:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncStigg) -> None:
        entitlement = await async_client.v1.addons.entitlements.create(
            addon_id="addonId",
            entitlements=[
                {
                    "id": "id",
                    "type": "FEATURE",
                }
            ],
        )
        assert_matches_type(EntitlementCreateResponse, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.entitlements.with_raw_response.create(
            addon_id="addonId",
            entitlements=[
                {
                    "id": "id",
                    "type": "FEATURE",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = await response.parse()
        assert_matches_type(EntitlementCreateResponse, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.entitlements.with_streaming_response.create(
            addon_id="addonId",
            entitlements=[
                {
                    "id": "id",
                    "type": "FEATURE",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = await response.parse()
            assert_matches_type(EntitlementCreateResponse, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            await async_client.v1.addons.entitlements.with_raw_response.create(
                addon_id="",
                entitlements=[
                    {
                        "id": "id",
                        "type": "FEATURE",
                    }
                ],
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_overload_1(self, async_client: AsyncStigg) -> None:
        entitlement = await async_client.v1.addons.entitlements.update(
            id="id",
            addon_id="addonId",
            type="FEATURE",
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params_overload_1(self, async_client: AsyncStigg) -> None:
        entitlement = await async_client.v1.addons.entitlements.update(
            id="id",
            addon_id="addonId",
            type="FEATURE",
            behavior="Increment",
            description="description",
            display_name_override="displayNameOverride",
            enum_values=["string"],
            has_soft_limit=True,
            has_unlimited_usage=True,
            hidden_from_widgets=["PAYWALL"],
            is_custom=True,
            is_granted=True,
            monthly_reset_period_configuration={"according_to": "SubscriptionStart"},
            order=0,
            reset_period="YEAR",
            usage_limit=0,
            weekly_reset_period_configuration={"according_to": "SubscriptionStart"},
            yearly_reset_period_configuration={"according_to": "SubscriptionStart"},
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_overload_1(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.entitlements.with_raw_response.update(
            id="id",
            addon_id="addonId",
            type="FEATURE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = await response.parse()
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_overload_1(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.entitlements.with_streaming_response.update(
            id="id",
            addon_id="addonId",
            type="FEATURE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = await response.parse()
            assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_overload_1(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            await async_client.v1.addons.entitlements.with_raw_response.update(
                id="id",
                addon_id="",
                type="FEATURE",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.entitlements.with_raw_response.update(
                id="",
                addon_id="addonId",
                type="FEATURE",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_overload_2(self, async_client: AsyncStigg) -> None:
        entitlement = await async_client.v1.addons.entitlements.update(
            id="id",
            addon_id="addonId",
            type="CREDIT",
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params_overload_2(self, async_client: AsyncStigg) -> None:
        entitlement = await async_client.v1.addons.entitlements.update(
            id="id",
            addon_id="addonId",
            type="CREDIT",
            amount=1,
            behavior="Increment",
            cadence="MONTH",
            description="description",
            display_name_override="displayNameOverride",
            hidden_from_widgets=["PAYWALL"],
            is_custom=True,
            is_granted=True,
            order=0,
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_overload_2(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.entitlements.with_raw_response.update(
            id="id",
            addon_id="addonId",
            type="CREDIT",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = await response.parse()
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_overload_2(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.entitlements.with_streaming_response.update(
            id="id",
            addon_id="addonId",
            type="CREDIT",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = await response.parse()
            assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_overload_2(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            await async_client.v1.addons.entitlements.with_raw_response.update(
                id="id",
                addon_id="",
                type="CREDIT",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.entitlements.with_raw_response.update(
                id="",
                addon_id="addonId",
                type="CREDIT",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncStigg) -> None:
        entitlement = await async_client.v1.addons.entitlements.list(
            "addonId",
        )
        assert_matches_type(EntitlementListResponse, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.entitlements.with_raw_response.list(
            "addonId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = await response.parse()
        assert_matches_type(EntitlementListResponse, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.entitlements.with_streaming_response.list(
            "addonId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = await response.parse()
            assert_matches_type(EntitlementListResponse, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            await async_client.v1.addons.entitlements.with_raw_response.list(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncStigg) -> None:
        entitlement = await async_client.v1.addons.entitlements.delete(
            id="id",
            addon_id="addonId",
        )
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.entitlements.with_raw_response.delete(
            id="id",
            addon_id="addonId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        entitlement = await response.parse()
        assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.entitlements.with_streaming_response.delete(
            id="id",
            addon_id="addonId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            entitlement = await response.parse()
            assert_matches_type(AddonPackageEntitlement, entitlement, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `addon_id` but received ''"):
            await async_client.v1.addons.entitlements.with_raw_response.delete(
                id="id",
                addon_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.entitlements.with_raw_response.delete(
                id="",
                addon_id="addonId",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ProductUpdateProductParams", "ProductSettings", "UsageResetCutoffRule"]


class ProductUpdateProductParams(TypedDict, total=False):
    description: Optional[str]
    """Description of the product"""

    display_name: Annotated[str, PropertyInfo(alias="displayName")]
    """Display name of the product"""

    metadata: Optional[Dict[str, str]]
    """Additional metadata for the product"""

    multiple_subscriptions: Annotated[bool, PropertyInfo(alias="multipleSubscriptions")]
    """Indicates if multiple subscriptions to this product are allowed"""

    product_settings: Annotated[ProductSettings, PropertyInfo(alias="productSettings")]

    usage_reset_cutoff_rule: Annotated[UsageResetCutoffRule, PropertyInfo(alias="usageResetCutoffRule")]
    """Rule defining when usage resets upon subscription update."""


class ProductSettings(TypedDict, total=False):
    subscription_cancellation_time: Required[
        Annotated[
            Literal["END_OF_BILLING_PERIOD", "IMMEDIATE", "SPECIFIC_DATE"],
            PropertyInfo(alias="subscriptionCancellationTime"),
        ]
    ]
    """Time when the subscription will be cancelled"""

    subscription_end_setup: Required[
        Annotated[Literal["DOWNGRADE_TO_FREE", "CANCEL_SUBSCRIPTION"], PropertyInfo(alias="subscriptionEndSetup")]
    ]
    """Setup for the end of the subscription"""

    subscription_start_setup: Required[
        Annotated[Literal["PLAN_SELECTION", "TRIAL_PERIOD", "FREE_PLAN"], PropertyInfo(alias="subscriptionStartSetup")]
    ]
    """Setup for the start of the subscription"""

    downgrade_plan_id: Annotated[Optional[str], PropertyInfo(alias="downgradePlanId")]
    """ID of the plan to downgrade to at the end of the billing period"""

    prorate_at_end_of_billing_period: Annotated[Optional[bool], PropertyInfo(alias="prorateAtEndOfBillingPeriod")]

    subscription_start_plan_id: Annotated[Optional[str], PropertyInfo(alias="subscriptionStartPlanId")]
    """ID of the plan to start the subscription with"""


class UsageResetCutoffRule(TypedDict, total=False):
    """Rule defining when usage resets upon subscription update."""

    behavior: Required[Literal["NEVER_RESET", "ALWAYS_RESET", "BILLING_PERIOD_CHANGE"]]
    """Behavior of the usage reset cutoff rule"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.v1.events.credits import grant_list_params, grant_create_params
from .....types.v1.events.credits.grant_list_response import GrantListResponse
from .....types.v1.events.credits.credit_grant_response import CreditGrantResponse

__all__ = ["GrantsResource", "AsyncGrantsResource"]


class GrantsResource(SyncAPIResource):
    """Operations related to credit grants"""

    @cached_property
    def with_raw_response(self) -> GrantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return GrantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GrantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return GrantsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        amount: float,
        currency_id: str,
        customer_id: str,
        display_name: str,
        grant_type: Literal["PAID", "PROMOTIONAL", "RECURRING"],
        await_payment_confirmation: bool | Omit = omit,
        billing_information: grant_create_params.BillingInformation | Omit = omit,
        comment: str | Omit = omit,
        cost: grant_create_params.Cost | Omit = omit,
        effective_at: Union[str, datetime] | Omit = omit,
        expire_at: Union[str, datetime] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        payment_collection_method: Literal["CHARGE", "INVOICE", "NONE"] | Omit = omit,
        priority: int | Omit = omit,
        resource_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGrantResponse:
        """
        Creates a new credit grant for a customer with specified amount, type, and
        optional billing configuration.

        Args:
          amount: The credit amount to grant

          currency_id: The credit currency ID (required)

          customer_id: The customer ID to grant credits to (required)

          display_name: The display name for the credit grant

          grant_type: The type of credit grant (PAID, PROMOTIONAL, RECURRING)

          await_payment_confirmation: Whether to wait for payment confirmation before returning (default: true)

          billing_information: Billing information for the credit grant

          comment: An optional comment on the credit grant

          cost: The monetary cost of the credit grant

          effective_at: The date when the credit grant becomes effective

          expire_at: The date when the credit grant expires

          metadata: Additional metadata for the credit grant

          payment_collection_method: The payment collection method (CHARGE, INVOICE, NONE)

          priority: The priority of the credit grant (lower number = higher priority)

          resource_id: The resource ID to scope the grant to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/credits/grants",
            body=maybe_transform(
                {
                    "amount": amount,
                    "currency_id": currency_id,
                    "customer_id": customer_id,
                    "display_name": display_name,
                    "grant_type": grant_type,
                    "await_payment_confirmation": await_payment_confirmation,
                    "billing_information": billing_information,
                    "comment": comment,
                    "cost": cost,
                    "effective_at": effective_at,
                    "expire_at": expire_at,
                    "metadata": metadata,
                    "payment_collection_method": payment_collection_method,
                    "priority": priority,
                    "resource_id": resource_id,
                },
                grant_create_params.GrantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantResponse,
        )

    def list(
        self,
        *,
        customer_id: str,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: grant_list_params.CreatedAt | Omit = omit,
        currency_id: str | Omit = omit,
        limit: int | Omit = omit,
        resource_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncMyCursorIDPage[GrantListResponse]:
        """
        Retrieves a paginated list of credit grants for a customer.

        Args:
          customer_id: Filter by customer ID (required)

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          currency_id: Filter by currency ID

          limit: Maximum number of items to return

          resource_id: Filter by resource ID. When omitted, only grants without a resource are returned

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/credits/grants",
            page=SyncMyCursorIDPage[GrantListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_id": customer_id,
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "currency_id": currency_id,
                        "limit": limit,
                        "resource_id": resource_id,
                    },
                    grant_list_params.GrantListParams,
                ),
            ),
            model=GrantListResponse,
        )

    def void(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGrantResponse:
        """
        Voids an existing credit grant, preventing further consumption of the remaining
        credits.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/credits/grants/{id}/void",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantResponse,
        )


class AsyncGrantsResource(AsyncAPIResource):
    """Operations related to credit grants"""

    @cached_property
    def with_raw_response(self) -> AsyncGrantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGrantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGrantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncGrantsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        amount: float,
        currency_id: str,
        customer_id: str,
        display_name: str,
        grant_type: Literal["PAID", "PROMOTIONAL", "RECURRING"],
        await_payment_confirmation: bool | Omit = omit,
        billing_information: grant_create_params.BillingInformation | Omit = omit,
        comment: str | Omit = omit,
        cost: grant_create_params.Cost | Omit = omit,
        effective_at: Union[str, datetime] | Omit = omit,
        expire_at: Union[str, datetime] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        payment_collection_method: Literal["CHARGE", "INVOICE", "NONE"] | Omit = omit,
        priority: int | Omit = omit,
        resource_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGrantResponse:
        """
        Creates a new credit grant for a customer with specified amount, type, and
        optional billing configuration.

        Args:
          amount: The credit amount to grant

          currency_id: The credit currency ID (required)

          customer_id: The customer ID to grant credits to (required)

          display_name: The display name for the credit grant

          grant_type: The type of credit grant (PAID, PROMOTIONAL, RECURRING)

          await_payment_confirmation: Whether to wait for payment confirmation before returning (default: true)

          billing_information: Billing information for the credit grant

          comment: An optional comment on the credit grant

          cost: The monetary cost of the credit grant

          effective_at: The date when the credit grant becomes effective

          expire_at: The date when the credit grant expires

          metadata: Additional metadata for the credit grant

          payment_collection_method: The payment collection method (CHARGE, INVOICE, NONE)

          priority: The priority of the credit grant (lower number = higher priority)

          resource_id: The resource ID to scope the grant to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/credits/grants",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "currency_id": currency_id,
                    "customer_id": customer_id,
                    "display_name": display_name,
                    "grant_type": grant_type,
                    "await_payment_confirmation": await_payment_confirmation,
                    "billing_information": billing_information,
                    "comment": comment,
                    "cost": cost,
                    "effective_at": effective_at,
                    "expire_at": expire_at,
                    "metadata": metadata,
                    "payment_collection_method": payment_collection_method,
                    "priority": priority,
                    "resource_id": resource_id,
                },
                grant_create_params.GrantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantResponse,
        )

    def list(
        self,
        *,
        customer_id: str,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: grant_list_params.CreatedAt | Omit = omit,
        currency_id: str | Omit = omit,
        limit: int | Omit = omit,
        resource_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[GrantListResponse, AsyncMyCursorIDPage[GrantListResponse]]:
        """
        Retrieves a paginated list of credit grants for a customer.

        Args:
          customer_id: Filter by customer ID (required)

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          currency_id: Filter by currency ID

          limit: Maximum number of items to return

          resource_id: Filter by resource ID. When omitted, only grants without a resource are returned

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/credits/grants",
            page=AsyncMyCursorIDPage[GrantListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_id": customer_id,
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "currency_id": currency_id,
                        "limit": limit,
                        "resource_id": resource_id,
                    },
                    grant_list_params.GrantListParams,
                ),
            ),
            model=GrantListResponse,
        )

    async def void(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGrantResponse:
        """
        Voids an existing credit grant, preventing further consumption of the remaining
        credits.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/credits/grants/{id}/void",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGrantResponse,
        )


class GrantsResourceWithRawResponse:
    def __init__(self, grants: GrantsResource) -> None:
        self._grants = grants

        self.create = to_raw_response_wrapper(
            grants.create,
        )
        self.list = to_raw_response_wrapper(
            grants.list,
        )
        self.void = to_raw_response_wrapper(
            grants.void,
        )


class AsyncGrantsResourceWithRawResponse:
    def __init__(self, grants: AsyncGrantsResource) -> None:
        self._grants = grants

        self.create = async_to_raw_response_wrapper(
            grants.create,
        )
        self.list = async_to_raw_response_wrapper(
            grants.list,
        )
        self.void = async_to_raw_response_wrapper(
            grants.void,
        )


class GrantsResourceWithStreamingResponse:
    def __init__(self, grants: GrantsResource) -> None:
        self._grants = grants

        self.create = to_streamed_response_wrapper(
            grants.create,
        )
        self.list = to_streamed_response_wrapper(
            grants.list,
        )
        self.void = to_streamed_response_wrapper(
            grants.void,
        )


class AsyncGrantsResourceWithStreamingResponse:
    def __init__(self, grants: AsyncGrantsResource) -> None:
        self._grants = grants

        self.create = async_to_streamed_response_wrapper(
            grants.create,
        )
        self.list = async_to_streamed_response_wrapper(
            grants.list,
        )
        self.void = async_to_streamed_response_wrapper(
            grants.void,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CreditListLedgerParams"]


class CreditListLedgerParams(TypedDict, total=False):
    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """Filter by customer ID (required)"""

    after: str
    """Return items that come after this cursor"""

    before: str
    """Return items that come before this cursor"""

    currency_id: Annotated[str, PropertyInfo(alias="currencyId")]
    """Filter by currency ID"""

    limit: int
    """Maximum number of items to return"""

    resource_id: Annotated[str, PropertyInfo(alias="resourceId")]
    """Filter by resource ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CreditGetUsageParams"]


class CreditGetUsageParams(TypedDict, total=False):
    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """Filter by customer ID (required)"""

    currency_id: Annotated[str, PropertyInfo(alias="currencyId")]
    """Filter by currency ID"""

    resource_id: Annotated[str, PropertyInfo(alias="resourceId")]
    """Filter by resource ID"""

    time_range: Annotated[Literal["LAST_DAY", "LAST_WEEK", "LAST_MONTH", "LAST_YEAR"], PropertyInfo(alias="timeRange")]
    """Time range for usage data (LAST_DAY, LAST_WEEK, LAST_MONTH, LAST_YEAR).

    Defaults to LAST_MONTH
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["GrantListParams", "CreatedAt"]


class GrantListParams(TypedDict, total=False):
    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """Filter by customer ID (required)"""

    after: str
    """Return items that come after this cursor"""

    before: str
    """Return items that come before this cursor"""

    created_at: Annotated[CreatedAt, PropertyInfo(alias="createdAt")]
    """Filter by creation date using range operators: gt, gte, lt, lte"""

    currency_id: Annotated[str, PropertyInfo(alias="currencyId")]
    """Filter by currency ID"""

    limit: int
    """Maximum number of items to return"""

    resource_id: Annotated[str, PropertyInfo(alias="resourceId")]
    """Filter by resource ID.

    When omitted, only grants without a resource are returned
    """


class CreatedAt(TypedDict, total=False):
    """Filter by creation date using range operators: gt, gte, lt, lte"""

    gt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Greater than the specified createdAt value"""

    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Greater than or equal to the specified createdAt value"""

    lt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Less than the specified createdAt value"""

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Less than or equal to the specified createdAt value"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .credit_get_usage_params import CreditGetUsageParams as CreditGetUsageParams
from .credit_get_usage_response import CreditGetUsageResponse as CreditGetUsageResponse
from .credit_list_ledger_params import CreditListLedgerParams as CreditListLedgerParams
from .credit_list_ledger_response import CreditListLedgerResponse as CreditListLedgerResponse
from .credit_get_auto_recharge_params import CreditGetAutoRechargeParams as CreditGetAutoRechargeParams
from .credit_get_auto_recharge_response import CreditGetAutoRechargeResponse as CreditGetAutoRechargeResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["FeatureCreateFeatureParams", "EnumConfiguration", "UnitTransformation"]


class FeatureCreateFeatureParams(TypedDict, total=False):
    id: Required[str]
    """The unique identifier for the feature"""

    display_name: Required[Annotated[str, PropertyInfo(alias="displayName")]]
    """The display name for the feature"""

    feature_type: Required[Annotated[Literal["BOOLEAN", "NUMBER", "ENUM"], PropertyInfo(alias="featureType")]]
    """The type of the feature"""

    description: str
    """The description for the feature"""

    enum_configuration: Annotated[Iterable[EnumConfiguration], PropertyInfo(alias="enumConfiguration")]
    """The configuration data for the feature"""

    feature_status: Annotated[Literal["NEW", "SUSPENDED", "ACTIVE"], PropertyInfo(alias="featureStatus")]
    """The status of the feature"""

    feature_units: Annotated[str, PropertyInfo(alias="featureUnits")]
    """The units for the feature"""

    feature_units_plural: Annotated[str, PropertyInfo(alias="featureUnitsPlural")]
    """The plural units for the feature"""

    metadata: Dict[str, str]
    """The additional metadata for the feature"""

    meter_type: Annotated[Literal["None", "FLUCTUATING", "INCREMENTAL"], PropertyInfo(alias="meterType")]
    """The meter type for the feature"""

    unit_transformation: Annotated[Optional[UnitTransformation], PropertyInfo(alias="unitTransformation")]
    """Unit transformation to be applied to the reported usage"""


class EnumConfiguration(TypedDict, total=False):
    display_name: Required[Annotated[str, PropertyInfo(alias="displayName")]]
    """The display name for the enum configuration entity"""

    value: Required[str]
    """The unique value identifier for the enum configuration entity"""


class UnitTransformation(TypedDict, total=False):
    """Unit transformation to be applied to the reported usage"""

    divide: Required[int]
    """Divide usage by this number"""

    feature_units: Annotated[str, PropertyInfo(alias="featureUnits")]
    """Singular feature units after the transformation"""

    feature_units_plural: Annotated[str, PropertyInfo(alias="featureUnitsPlural")]
    """Plural feature units after the transformation"""

    round: Literal["UP", "DOWN"]
    """After division, either round the result up or down"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CustomerImportResponse", "Data"]


class Data(BaseModel):
    """List of newly created customer IDs from the import operation."""

    new_customers: List[str] = FieldInfo(alias="newCustomers")
    """Customer IDs created during import"""


class CustomerImportResponse(BaseModel):
    """Response object"""

    data: Data
    """List of newly created customer IDs from the import operation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = [
    "PromotionalEntitlementListResponse",
    "ResetPeriodConfiguration",
    "ResetPeriodConfigurationYearlyResetPeriodConfig",
    "ResetPeriodConfigurationMonthlyResetPeriodConfig",
    "ResetPeriodConfigurationWeeklyResetPeriodConfig",
]


class ResetPeriodConfigurationYearlyResetPeriodConfig(BaseModel):
    """Yearly reset configuration"""

    according_to: Literal["SubscriptionStart"] = FieldInfo(alias="accordingTo")
    """Reset anchor (SubscriptionStart)"""


class ResetPeriodConfigurationMonthlyResetPeriodConfig(BaseModel):
    """Monthly reset configuration"""

    according_to: Literal["SubscriptionStart", "StartOfTheMonth"] = FieldInfo(alias="accordingTo")
    """Reset anchor (SubscriptionStart or StartOfTheMonth)"""


class ResetPeriodConfigurationWeeklyResetPeriodConfig(BaseModel):
    """Weekly reset configuration"""

    according_to: Literal[
        "SubscriptionStart",
        "EverySunday",
        "EveryMonday",
        "EveryTuesday",
        "EveryWednesday",
        "EveryThursday",
        "EveryFriday",
        "EverySaturday",
    ] = FieldInfo(alias="accordingTo")
    """Reset anchor (SubscriptionStart or specific day)"""


ResetPeriodConfiguration: TypeAlias = Union[
    ResetPeriodConfigurationYearlyResetPeriodConfig,
    ResetPeriodConfigurationMonthlyResetPeriodConfig,
    ResetPeriodConfigurationWeeklyResetPeriodConfig,
    None,
]


class PromotionalEntitlementListResponse(BaseModel):
    """Granted feature entitlement"""

    id: str
    """Unique identifier for the entity"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    description: Optional[str] = None
    """The description of the entitlement"""

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)
    """The end date of the promotional entitlement"""

    enum_values: Optional[List[str]] = FieldInfo(alias="enumValues", default=None)
    """The enum values of the entitlement"""

    environment_id: str = FieldInfo(alias="environmentId")
    """The unique identifier for the environment"""

    feature_group_ids: Optional[List[str]] = FieldInfo(alias="featureGroupIds", default=None)
    """Feature group IDs associated with this entitlement"""

    feature_id: str = FieldInfo(alias="featureId")
    """The unique identifier of the entitlement feature"""

    has_soft_limit: Optional[bool] = FieldInfo(alias="hasSoftLimit", default=None)
    """Whether the entitlement has a soft limit"""

    has_unlimited_usage: Optional[bool] = FieldInfo(alias="hasUnlimitedUsage", default=None)
    """Whether the entitlement has an unlimited usage"""

    is_visible: bool = FieldInfo(alias="isVisible")
    """Whether the entitlement is visible"""

    period: Literal["1 week", "1 month", "6 month", "1 year", "lifetime", "custom"]
    """The grant period of the promotional entitlement"""

    reset_period: Optional[Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"]] = FieldInfo(
        alias="resetPeriod", default=None
    )
    """The reset period of the entitlement"""

    reset_period_configuration: Optional[ResetPeriodConfiguration] = FieldInfo(
        alias="resetPeriodConfiguration", default=None
    )
    """The reset period configuration of the entitlement"""

    start_date: datetime = FieldInfo(alias="startDate")
    """The start date of the entitlement"""

    status: Literal["Active", "Expired", "Paused"]
    """The status of the entitlement"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""

    usage_limit: Optional[float] = FieldInfo(alias="usageLimit", default=None)
    """The usage limit of the entitlement"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PlanListParams", "CreatedAt"]


class PlanListParams(TypedDict, total=False):
    after: str
    """Return items that come after this cursor"""

    before: str
    """Return items that come before this cursor"""

    created_at: Annotated[CreatedAt, PropertyInfo(alias="createdAt")]
    """Filter by creation date using range operators: gt, gte, lt, lte"""

    limit: int
    """Maximum number of items to return"""

    product_id: Annotated[str, PropertyInfo(alias="productId")]
    """Filter by product ID"""

    status: str
    """Filter by status. Supports comma-separated values for multiple statuses"""


class CreatedAt(TypedDict, total=False):
    """Filter by creation date using range operators: gt, gte, lt, lte"""

    gt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Greater than the specified createdAt value"""

    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Greater than or equal to the specified createdAt value"""

    lt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Less than the specified createdAt value"""

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Less than or equal to the specified createdAt value"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["CreditGetAutoRechargeResponse", "Data"]


class Data(BaseModel):
    """Automatic recharge configuration for a customer and currency"""

    id: Optional[str] = None
    """The unique configuration ID"""

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)
    """Timestamp of when the record was created"""

    currency_id: str = FieldInfo(alias="currencyId")
    """The currency ID for this configuration"""

    customer_id: str = FieldInfo(alias="customerId")
    """The customer ID this configuration belongs to"""

    grant_expiration_period: Literal["1_MONTH", "1_YEAR"] = FieldInfo(alias="grantExpirationPeriod")
    """Expiration period for auto-recharge grants (1_MONTH or 1_YEAR)"""

    is_enabled: bool = FieldInfo(alias="isEnabled")
    """Whether automatic recharge is enabled"""

    max_spend_limit: Optional[float] = FieldInfo(alias="maxSpendLimit", default=None)
    """Maximum monthly spend limit for automatic recharges"""

    target_balance: float = FieldInfo(alias="targetBalance")
    """The target credit balance to recharge to"""

    threshold_type: Literal["CREDIT_AMOUNT", "DOLLAR_AMOUNT"] = FieldInfo(alias="thresholdType")
    """The threshold type (CREDIT_AMOUNT or DOLLAR_AMOUNT)"""

    threshold_value: float = FieldInfo(alias="thresholdValue")
    """The threshold value that triggers a recharge"""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)
    """Timestamp of when the record was last updated"""


class CreditGetAutoRechargeResponse(BaseModel):
    """Response object"""

    data: Data
    """Automatic recharge configuration for a customer and currency"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.customers import payment_method_attach_params
from ....types.v1.customer_response import CustomerResponse

__all__ = ["PaymentMethodResource", "AsyncPaymentMethodResource"]


class PaymentMethodResource(SyncAPIResource):
    """Operations related to customers"""

    @cached_property
    def with_raw_response(self) -> PaymentMethodResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return PaymentMethodResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PaymentMethodResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return PaymentMethodResourceWithStreamingResponse(self)

    def attach(
        self,
        id: str,
        *,
        integration_id: str,
        payment_method_id: str,
        vendor_identifier: Literal[
            "AUTH0",
            "ZUORA",
            "STRIPE",
            "HUBSPOT",
            "AWS_MARKETPLACE",
            "SNOWFLAKE",
            "SALESFORCE",
            "BIG_QUERY",
            "OPEN_FGA",
            "APP_STORE",
        ],
        billing_currency: Optional[
            Literal[
                "usd",
                "aed",
                "all",
                "amd",
                "ang",
                "aud",
                "awg",
                "azn",
                "bam",
                "bbd",
                "bdt",
                "bgn",
                "bif",
                "bmd",
                "bnd",
                "bsd",
                "bwp",
                "byn",
                "bzd",
                "brl",
                "cad",
                "cdf",
                "chf",
                "cny",
                "czk",
                "dkk",
                "dop",
                "dzd",
                "egp",
                "etb",
                "eur",
                "fjd",
                "gbp",
                "gel",
                "gip",
                "gmd",
                "gyd",
                "hkd",
                "hrk",
                "htg",
                "idr",
                "ils",
                "inr",
                "isk",
                "jmd",
                "jpy",
                "kes",
                "kgs",
                "khr",
                "kmf",
                "krw",
                "kyd",
                "kzt",
                "lbp",
                "lkr",
                "lrd",
                "lsl",
                "mad",
                "mdl",
                "mga",
                "mkd",
                "mmk",
                "mnt",
                "mop",
                "mro",
                "mvr",
                "mwk",
                "mxn",
                "myr",
                "mzn",
                "nad",
                "ngn",
                "nok",
                "npr",
                "nzd",
                "pgk",
                "php",
                "pkr",
                "pln",
                "qar",
                "ron",
                "rsd",
                "rub",
                "rwf",
                "sar",
                "sbd",
                "scr",
                "sek",
                "sgd",
                "sle",
                "sll",
                "sos",
                "szl",
                "thb",
                "tjs",
                "top",
                "try",
                "ttd",
                "tzs",
                "uah",
                "uzs",
                "vnd",
                "vuv",
                "wst",
                "xaf",
                "xcd",
                "yer",
                "zar",
                "zmw",
                "clp",
                "djf",
                "gnf",
                "ugx",
                "pyg",
                "xof",
                "xpf",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """Attaches a payment method to a customer for billing.

        Required for paid
        subscriptions when integrated with a billing provider.

        Args:
          integration_id: Integration details

          payment_method_id: Billing provider payment method id

          vendor_identifier: The vendor identifier of integration

          billing_currency: Customers selected currency

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/customers/{id}/payment-method",
            body=maybe_transform(
                {
                    "integration_id": integration_id,
                    "payment_method_id": payment_method_id,
                    "vendor_identifier": vendor_identifier,
                    "billing_currency": billing_currency,
                },
                payment_method_attach_params.PaymentMethodAttachParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    def detach(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """Removes the payment method from a customer.

        Ensure active paid subscriptions
        have an alternative payment method.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/api/v1/customers/{id}/payment-method",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )


class AsyncPaymentMethodResource(AsyncAPIResource):
    """Operations related to customers"""

    @cached_property
    def with_raw_response(self) -> AsyncPaymentMethodResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPaymentMethodResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPaymentMethodResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncPaymentMethodResourceWithStreamingResponse(self)

    async def attach(
        self,
        id: str,
        *,
        integration_id: str,
        payment_method_id: str,
        vendor_identifier: Literal[
            "AUTH0",
            "ZUORA",
            "STRIPE",
            "HUBSPOT",
            "AWS_MARKETPLACE",
            "SNOWFLAKE",
            "SALESFORCE",
            "BIG_QUERY",
            "OPEN_FGA",
            "APP_STORE",
        ],
        billing_currency: Optional[
            Literal[
                "usd",
                "aed",
                "all",
                "amd",
                "ang",
                "aud",
                "awg",
                "azn",
                "bam",
                "bbd",
                "bdt",
                "bgn",
                "bif",
                "bmd",
                "bnd",
                "bsd",
                "bwp",
                "byn",
                "bzd",
                "brl",
                "cad",
                "cdf",
                "chf",
                "cny",
                "czk",
                "dkk",
                "dop",
                "dzd",
                "egp",
                "etb",
                "eur",
                "fjd",
                "gbp",
                "gel",
                "gip",
                "gmd",
                "gyd",
                "hkd",
                "hrk",
                "htg",
                "idr",
                "ils",
                "inr",
                "isk",
                "jmd",
                "jpy",
                "kes",
                "kgs",
                "khr",
                "kmf",
                "krw",
                "kyd",
                "kzt",
                "lbp",
                "lkr",
                "lrd",
                "lsl",
                "mad",
                "mdl",
                "mga",
                "mkd",
                "mmk",
                "mnt",
                "mop",
                "mro",
                "mvr",
                "mwk",
                "mxn",
                "myr",
                "mzn",
                "nad",
                "ngn",
                "nok",
                "npr",
                "nzd",
                "pgk",
                "php",
                "pkr",
                "pln",
                "qar",
                "ron",
                "rsd",
                "rub",
                "rwf",
                "sar",
                "sbd",
                "scr",
                "sek",
                "sgd",
                "sle",
                "sll",
                "sos",
                "szl",
                "thb",
                "tjs",
                "top",
                "try",
                "ttd",
                "tzs",
                "uah",
                "uzs",
                "vnd",
                "vuv",
                "wst",
                "xaf",
                "xcd",
                "yer",
                "zar",
                "zmw",
                "clp",
                "djf",
                "gnf",
                "ugx",
                "pyg",
                "xof",
                "xpf",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """Attaches a payment method to a customer for billing.

        Required for paid
        subscriptions when integrated with a billing provider.

        Args:
          integration_id: Integration details

          payment_method_id: Billing provider payment method id

          vendor_identifier: The vendor identifier of integration

          billing_currency: Customers selected currency

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/customers/{id}/payment-method",
            body=await async_maybe_transform(
                {
                    "integration_id": integration_id,
                    "payment_method_id": payment_method_id,
                    "vendor_identifier": vendor_identifier,
                    "billing_currency": billing_currency,
                },
                payment_method_attach_params.PaymentMethodAttachParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    async def detach(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """Removes the payment method from a customer.

        Ensure active paid subscriptions
        have an alternative payment method.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/api/v1/customers/{id}/payment-method",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )


class PaymentMethodResourceWithRawResponse:
    def __init__(self, payment_method: PaymentMethodResource) -> None:
        self._payment_method = payment_method

        self.attach = to_raw_response_wrapper(
            payment_method.attach,
        )
        self.detach = to_raw_response_wrapper(
            payment_method.detach,
        )


class AsyncPaymentMethodResourceWithRawResponse:
    def __init__(self, payment_method: AsyncPaymentMethodResource) -> None:
        self._payment_method = payment_method

        self.attach = async_to_raw_response_wrapper(
            payment_method.attach,
        )
        self.detach = async_to_raw_response_wrapper(
            payment_method.detach,
        )


class PaymentMethodResourceWithStreamingResponse:
    def __init__(self, payment_method: PaymentMethodResource) -> None:
        self._payment_method = payment_method

        self.attach = to_streamed_response_wrapper(
            payment_method.attach,
        )
        self.detach = to_streamed_response_wrapper(
            payment_method.detach,
        )


class AsyncPaymentMethodResourceWithStreamingResponse:
    def __init__(self, payment_method: AsyncPaymentMethodResource) -> None:
        self._payment_method = payment_method

        self.attach = async_to_streamed_response_wrapper(
            payment_method.attach,
        )
        self.detach = async_to_streamed_response_wrapper(
            payment_method.detach,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = [
    "FeatureUpdateFeatureParams",
    "EnumConfiguration",
    "Meter",
    "MeterAggregation",
    "MeterFilter",
    "MeterFilterCondition",
    "UnitTransformation",
]


class FeatureUpdateFeatureParams(TypedDict, total=False):
    description: str
    """The description for the feature"""

    display_name: Annotated[str, PropertyInfo(alias="displayName")]
    """The display name for the feature"""

    enum_configuration: Annotated[Iterable[EnumConfiguration], PropertyInfo(alias="enumConfiguration")]
    """The configuration data for the feature"""

    feature_units: Annotated[str, PropertyInfo(alias="featureUnits")]
    """The units for the feature"""

    feature_units_plural: Annotated[str, PropertyInfo(alias="featureUnitsPlural")]
    """The plural units for the feature"""

    metadata: Dict[str, str]
    """The additional metadata for the feature"""

    meter: Meter

    unit_transformation: Annotated[Optional[UnitTransformation], PropertyInfo(alias="unitTransformation")]
    """Unit transformation to be applied to the reported usage"""


class EnumConfiguration(TypedDict, total=False):
    display_name: Required[Annotated[str, PropertyInfo(alias="displayName")]]
    """The display name for the enum configuration entity"""

    value: Required[str]
    """The unique value identifier for the enum configuration entity"""


class MeterAggregation(TypedDict, total=False):
    function: Required[Literal["SUM", "MAX", "MIN", "AVG", "COUNT", "UNIQUE"]]

    field: str


class MeterFilterCondition(TypedDict, total=False):
    field: Required[str]

    operation: Required[
        Literal[
            "EQUALS",
            "NOT_EQUALS",
            "GREATER_THAN",
            "GREATER_THAN_OR_EQUAL",
            "LESS_THAN",
            "LESS_THAN_OR_EQUAL",
            "IS_NULL",
            "IS_NOT_NULL",
            "CONTAINS",
            "STARTS_WITH",
            "ENDS_WITH",
            "IN",
        ]
    ]

    value: str

    values: SequenceNotStr[str]


class MeterFilter(TypedDict, total=False):
    conditions: Required[Iterable[MeterFilterCondition]]


class Meter(TypedDict, total=False):
    aggregation: Required[MeterAggregation]

    filters: Required[Iterable[MeterFilter]]


class UnitTransformation(TypedDict, total=False):
    """Unit transformation to be applied to the reported usage"""

    divide: Required[int]
    """Divide usage by this number"""

    feature_units: Annotated[str, PropertyInfo(alias="featureUnits")]
    """Singular feature units after the transformation"""

    feature_units_plural: Annotated[str, PropertyInfo(alias="featureUnitsPlural")]
    """Plural feature units after the transformation"""

    round: Literal["UP", "DOWN"]
    """After division, either round the result up or down"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .plan import Plan as Plan
from .addon import Addon as Addon
from .coupon import Coupon as Coupon
from .feature import Feature as Feature
from .product import Product as Product
from .subscription import Subscription as Subscription
from .plan_list_params import PlanListParams as PlanListParams
from .addon_list_params import AddonListParams as AddonListParams
from .customer_response import CustomerResponse as CustomerResponse
from .coupon_list_params import CouponListParams as CouponListParams
from .plan_create_params import PlanCreateParams as PlanCreateParams
from .plan_list_response import PlanListResponse as PlanListResponse
from .plan_update_params import PlanUpdateParams as PlanUpdateParams
from .addon_create_params import AddonCreateParams as AddonCreateParams
from .addon_list_response import AddonListResponse as AddonListResponse
from .addon_update_params import AddonUpdateParams as AddonUpdateParams
from .event_report_params import EventReportParams as EventReportParams
from .plan_publish_params import PlanPublishParams as PlanPublishParams
from .usage_report_params import UsageReportParams as UsageReportParams
from .addon_publish_params import AddonPublishParams as AddonPublishParams
from .coupon_create_params import CouponCreateParams as CouponCreateParams
from .coupon_list_response import CouponListResponse as CouponListResponse
from .customer_list_params import CustomerListParams as CustomerListParams
from .usage_history_params import UsageHistoryParams as UsageHistoryParams
from .event_report_response import EventReportResponse as EventReportResponse
from .plan_publish_response import PlanPublishResponse as PlanPublishResponse
from .usage_report_response import UsageReportResponse as UsageReportResponse
from .addon_publish_response import AddonPublishResponse as AddonPublishResponse
from .customer_import_params import CustomerImportParams as CustomerImportParams
from .customer_list_response import CustomerListResponse as CustomerListResponse
from .customer_update_params import CustomerUpdateParams as CustomerUpdateParams
from .usage_history_response import UsageHistoryResponse as UsageHistoryResponse
from .plan_set_pricing_params import PlanSetPricingParams as PlanSetPricingParams
from .addon_set_pricing_params import AddonSetPricingParams as AddonSetPricingParams
from .customer_import_response import CustomerImportResponse as CustomerImportResponse
from .subscription_list_params import SubscriptionListParams as SubscriptionListParams
from .customer_provision_params import CustomerProvisionParams as CustomerProvisionParams
from .plan_remove_draft_response import PlanRemoveDraftResponse as PlanRemoveDraftResponse
from .subscription_cancel_params import SubscriptionCancelParams as SubscriptionCancelParams
from .subscription_import_params import SubscriptionImportParams as SubscriptionImportParams
from .subscription_list_response import SubscriptionListResponse as SubscriptionListResponse
from .subscription_update_params import SubscriptionUpdateParams as SubscriptionUpdateParams
from .addon_remove_draft_response import AddonRemoveDraftResponse as AddonRemoveDraftResponse
from .coupon_update_coupon_params import CouponUpdateCouponParams as CouponUpdateCouponParams
from .subscription_migrate_params import SubscriptionMigrateParams as SubscriptionMigrateParams
from .subscription_preview_params import SubscriptionPreviewParams as SubscriptionPreviewParams
from .feature_list_features_params import FeatureListFeaturesParams as FeatureListFeaturesParams
from .product_list_products_params import ProductListProductsParams as ProductListProductsParams
from .set_package_pricing_response import SetPackagePricingResponse as SetPackagePricingResponse
from .subscription_delegate_params import SubscriptionDelegateParams as SubscriptionDelegateParams
from .subscription_import_response import SubscriptionImportResponse as SubscriptionImportResponse
from .subscription_transfer_params import SubscriptionTransferParams as SubscriptionTransferParams
from .feature_create_feature_params import FeatureCreateFeatureParams as FeatureCreateFeatureParams
from .feature_update_feature_params import FeatureUpdateFeatureParams as FeatureUpdateFeatureParams
from .product_create_product_params import ProductCreateProductParams as ProductCreateProductParams
from .product_update_product_params import ProductUpdateProductParams as ProductUpdateProductParams
from .subscription_preview_response import SubscriptionPreviewResponse as SubscriptionPreviewResponse
from .subscription_provision_params import SubscriptionProvisionParams as SubscriptionProvisionParams
from .customer_list_resources_params import CustomerListResourcesParams as CustomerListResourcesParams
from .feature_list_features_response import FeatureListFeaturesResponse as FeatureListFeaturesResponse
from .product_list_products_response import ProductListProductsResponse as ProductListProductsResponse
from .subscription_provision_response import SubscriptionProvisionResponse as SubscriptionProvisionResponse
from .customer_list_resources_response import CustomerListResourcesResponse as CustomerListResourcesResponse
from .product_duplicate_product_params import ProductDuplicateProductParams as ProductDuplicateProductParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["UsageSyncResponse", "Data"]


class Data(BaseModel):
    """Result of triggering a subscription usage sync."""

    triggered: bool
    """Whether usage was synced to the billing provider"""


class UsageSyncResponse(BaseModel):
    """Response object"""

    data: Data
    """Result of triggering a subscription usage sync."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, Optional
from typing_extensions import Literal, overload

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import required_args, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.plans import entitlement_create_params, entitlement_update_params
from ....types.v1.plans.plan_entitlement import PlanEntitlement
from ....types.v1.plans.entitlement_list_response import EntitlementListResponse
from ....types.v1.plans.entitlement_create_response import EntitlementCreateResponse

__all__ = ["EntitlementsResource", "AsyncEntitlementsResource"]


class EntitlementsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EntitlementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return EntitlementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EntitlementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return EntitlementsResourceWithStreamingResponse(self)

    def create(
        self,
        plan_id: str,
        *,
        entitlements: Iterable[entitlement_create_params.Entitlement],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EntitlementCreateResponse:
        """
        Creates one or more entitlements (feature or credit) on a draft plan.

        Args:
          entitlements: Entitlements to create

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return self._post(
            f"/api/v1/plans/{plan_id}/entitlements",
            body=maybe_transform({"entitlements": entitlements}, entitlement_create_params.EntitlementCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EntitlementCreateResponse,
        )

    @overload
    def update(
        self,
        id: str,
        *,
        plan_id: str,
        type: Literal["FEATURE"],
        behavior: Literal["Increment", "Override"] | Omit = omit,
        description: str | Omit = omit,
        display_name_override: str | Omit = omit,
        enum_values: SequenceNotStr[str] | Omit = omit,
        has_soft_limit: bool | Omit = omit,
        has_unlimited_usage: bool | Omit = omit,
        hidden_from_widgets: List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]] | Omit = omit,
        is_custom: bool | Omit = omit,
        is_granted: bool | Omit = omit,
        monthly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestMonthlyResetPeriodConfiguration
        ]
        | Omit = omit,
        order: float | Omit = omit,
        reset_period: Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"] | Omit = omit,
        usage_limit: Optional[int] | Omit = omit,
        weekly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestWeeklyResetPeriodConfiguration
        ]
        | Omit = omit,
        yearly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestYearlyResetPeriodConfiguration
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanEntitlement:
        """
        Updates an existing entitlement on a draft plan.

        Args:
          type: UpdateFeatureEntitlementRequest

          behavior: Entitlement behavior (Increment or Override)

          description: Description of the entitlement

          display_name_override: Override display name for the entitlement

          enum_values: Allowed enum values for the feature entitlement

          has_soft_limit: Whether the usage limit is a soft limit

          has_unlimited_usage: Whether usage is unlimited

          hidden_from_widgets: Widget types where this entitlement is hidden

          is_custom: Whether this is a custom entitlement

          is_granted: Whether the entitlement is granted

          monthly_reset_period_configuration: Configuration for monthly reset period

          order: Display order of the entitlement

          reset_period: Period at which usage resets

          usage_limit: Maximum allowed usage for the feature

          weekly_reset_period_configuration: Configuration for weekly reset period

          yearly_reset_period_configuration: Configuration for yearly reset period

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        id: str,
        *,
        plan_id: str,
        type: Literal["CREDIT"],
        amount: float | Omit = omit,
        behavior: Literal["Increment", "Override"] | Omit = omit,
        cadence: Literal["MONTH", "YEAR"] | Omit = omit,
        description: str | Omit = omit,
        display_name_override: str | Omit = omit,
        hidden_from_widgets: List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]] | Omit = omit,
        is_custom: bool | Omit = omit,
        is_granted: bool | Omit = omit,
        order: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanEntitlement:
        """
        Updates an existing entitlement on a draft plan.

        Args:
          type: UpdateCreditEntitlementRequest

          amount: Credit grant amount

          behavior: Entitlement behavior (Increment or Override)

          cadence: Credit grant cadence (MONTH or YEAR)

          description: Description of the entitlement

          display_name_override: Override display name for the entitlement

          hidden_from_widgets: Widget types where this entitlement is hidden

          is_custom: Whether this is a custom entitlement

          is_granted: Whether the entitlement is granted

          order: Display order of the entitlement

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["plan_id", "type"])
    def update(
        self,
        id: str,
        *,
        plan_id: str,
        type: Literal["FEATURE"] | Literal["CREDIT"],
        behavior: Literal["Increment", "Override"] | Omit = omit,
        description: str | Omit = omit,
        display_name_override: str | Omit = omit,
        enum_values: SequenceNotStr[str] | Omit = omit,
        has_soft_limit: bool | Omit = omit,
        has_unlimited_usage: bool | Omit = omit,
        hidden_from_widgets: List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]] | Omit = omit,
        is_custom: bool | Omit = omit,
        is_granted: bool | Omit = omit,
        monthly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestMonthlyResetPeriodConfiguration
        ]
        | Omit = omit,
        order: float | Omit = omit,
        reset_period: Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"] | Omit = omit,
        usage_limit: Optional[int] | Omit = omit,
        weekly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestWeeklyResetPeriodConfiguration
        ]
        | Omit = omit,
        yearly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestYearlyResetPeriodConfiguration
        ]
        | Omit = omit,
        amount: float | Omit = omit,
        cadence: Literal["MONTH", "YEAR"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanEntitlement:
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/v1/plans/{plan_id}/entitlements/{id}",
            body=maybe_transform(
                {
                    "type": type,
                    "behavior": behavior,
                    "description": description,
                    "display_name_override": display_name_override,
                    "enum_values": enum_values,
                    "has_soft_limit": has_soft_limit,
                    "has_unlimited_usage": has_unlimited_usage,
                    "hidden_from_widgets": hidden_from_widgets,
                    "is_custom": is_custom,
                    "is_granted": is_granted,
                    "monthly_reset_period_configuration": monthly_reset_period_configuration,
                    "order": order,
                    "reset_period": reset_period,
                    "usage_limit": usage_limit,
                    "weekly_reset_period_configuration": weekly_reset_period_configuration,
                    "yearly_reset_period_configuration": yearly_reset_period_configuration,
                    "amount": amount,
                    "cadence": cadence,
                },
                entitlement_update_params.EntitlementUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanEntitlement,
        )

    def list(
        self,
        plan_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EntitlementListResponse:
        """
        Retrieves a list of entitlements for a plan.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return self._get(
            f"/api/v1/plans/{plan_id}/entitlements",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EntitlementListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        plan_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanEntitlement:
        """
        Deletes an entitlement from a draft plan.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/api/v1/plans/{plan_id}/entitlements/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanEntitlement,
        )


class AsyncEntitlementsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEntitlementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEntitlementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEntitlementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncEntitlementsResourceWithStreamingResponse(self)

    async def create(
        self,
        plan_id: str,
        *,
        entitlements: Iterable[entitlement_create_params.Entitlement],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EntitlementCreateResponse:
        """
        Creates one or more entitlements (feature or credit) on a draft plan.

        Args:
          entitlements: Entitlements to create

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return await self._post(
            f"/api/v1/plans/{plan_id}/entitlements",
            body=await async_maybe_transform(
                {"entitlements": entitlements}, entitlement_create_params.EntitlementCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EntitlementCreateResponse,
        )

    @overload
    async def update(
        self,
        id: str,
        *,
        plan_id: str,
        type: Literal["FEATURE"],
        behavior: Literal["Increment", "Override"] | Omit = omit,
        description: str | Omit = omit,
        display_name_override: str | Omit = omit,
        enum_values: SequenceNotStr[str] | Omit = omit,
        has_soft_limit: bool | Omit = omit,
        has_unlimited_usage: bool | Omit = omit,
        hidden_from_widgets: List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]] | Omit = omit,
        is_custom: bool | Omit = omit,
        is_granted: bool | Omit = omit,
        monthly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestMonthlyResetPeriodConfiguration
        ]
        | Omit = omit,
        order: float | Omit = omit,
        reset_period: Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"] | Omit = omit,
        usage_limit: Optional[int] | Omit = omit,
        weekly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestWeeklyResetPeriodConfiguration
        ]
        | Omit = omit,
        yearly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestYearlyResetPeriodConfiguration
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanEntitlement:
        """
        Updates an existing entitlement on a draft plan.

        Args:
          type: UpdateFeatureEntitlementRequest

          behavior: Entitlement behavior (Increment or Override)

          description: Description of the entitlement

          display_name_override: Override display name for the entitlement

          enum_values: Allowed enum values for the feature entitlement

          has_soft_limit: Whether the usage limit is a soft limit

          has_unlimited_usage: Whether usage is unlimited

          hidden_from_widgets: Widget types where this entitlement is hidden

          is_custom: Whether this is a custom entitlement

          is_granted: Whether the entitlement is granted

          monthly_reset_period_configuration: Configuration for monthly reset period

          order: Display order of the entitlement

          reset_period: Period at which usage resets

          usage_limit: Maximum allowed usage for the feature

          weekly_reset_period_configuration: Configuration for weekly reset period

          yearly_reset_period_configuration: Configuration for yearly reset period

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        id: str,
        *,
        plan_id: str,
        type: Literal["CREDIT"],
        amount: float | Omit = omit,
        behavior: Literal["Increment", "Override"] | Omit = omit,
        cadence: Literal["MONTH", "YEAR"] | Omit = omit,
        description: str | Omit = omit,
        display_name_override: str | Omit = omit,
        hidden_from_widgets: List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]] | Omit = omit,
        is_custom: bool | Omit = omit,
        is_granted: bool | Omit = omit,
        order: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanEntitlement:
        """
        Updates an existing entitlement on a draft plan.

        Args:
          type: UpdateCreditEntitlementRequest

          amount: Credit grant amount

          behavior: Entitlement behavior (Increment or Override)

          cadence: Credit grant cadence (MONTH or YEAR)

          description: Description of the entitlement

          display_name_override: Override display name for the entitlement

          hidden_from_widgets: Widget types where this entitlement is hidden

          is_custom: Whether this is a custom entitlement

          is_granted: Whether the entitlement is granted

          order: Display order of the entitlement

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["plan_id", "type"])
    async def update(
        self,
        id: str,
        *,
        plan_id: str,
        type: Literal["FEATURE"] | Literal["CREDIT"],
        behavior: Literal["Increment", "Override"] | Omit = omit,
        description: str | Omit = omit,
        display_name_override: str | Omit = omit,
        enum_values: SequenceNotStr[str] | Omit = omit,
        has_soft_limit: bool | Omit = omit,
        has_unlimited_usage: bool | Omit = omit,
        hidden_from_widgets: List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]] | Omit = omit,
        is_custom: bool | Omit = omit,
        is_granted: bool | Omit = omit,
        monthly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestMonthlyResetPeriodConfiguration
        ]
        | Omit = omit,
        order: float | Omit = omit,
        reset_period: Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"] | Omit = omit,
        usage_limit: Optional[int] | Omit = omit,
        weekly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestWeeklyResetPeriodConfiguration
        ]
        | Omit = omit,
        yearly_reset_period_configuration: Optional[
            entitlement_update_params.UpdateFeatureEntitlementRequestYearlyResetPeriodConfiguration
        ]
        | Omit = omit,
        amount: float | Omit = omit,
        cadence: Literal["MONTH", "YEAR"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanEntitlement:
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/v1/plans/{plan_id}/entitlements/{id}",
            body=await async_maybe_transform(
                {
                    "type": type,
                    "behavior": behavior,
                    "description": description,
                    "display_name_override": display_name_override,
                    "enum_values": enum_values,
                    "has_soft_limit": has_soft_limit,
                    "has_unlimited_usage": has_unlimited_usage,
                    "hidden_from_widgets": hidden_from_widgets,
                    "is_custom": is_custom,
                    "is_granted": is_granted,
                    "monthly_reset_period_configuration": monthly_reset_period_configuration,
                    "order": order,
                    "reset_period": reset_period,
                    "usage_limit": usage_limit,
                    "weekly_reset_period_configuration": weekly_reset_period_configuration,
                    "yearly_reset_period_configuration": yearly_reset_period_configuration,
                    "amount": amount,
                    "cadence": cadence,
                },
                entitlement_update_params.EntitlementUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanEntitlement,
        )

    async def list(
        self,
        plan_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EntitlementListResponse:
        """
        Retrieves a list of entitlements for a plan.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return await self._get(
            f"/api/v1/plans/{plan_id}/entitlements",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EntitlementListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        plan_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanEntitlement:
        """
        Deletes an entitlement from a draft plan.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/api/v1/plans/{plan_id}/entitlements/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanEntitlement,
        )


class EntitlementsResourceWithRawResponse:
    def __init__(self, entitlements: EntitlementsResource) -> None:
        self._entitlements = entitlements

        self.create = to_raw_response_wrapper(
            entitlements.create,
        )
        self.update = to_raw_response_wrapper(
            entitlements.update,
        )
        self.list = to_raw_response_wrapper(
            entitlements.list,
        )
        self.delete = to_raw_response_wrapper(
            entitlements.delete,
        )


class AsyncEntitlementsResourceWithRawResponse:
    def __init__(self, entitlements: AsyncEntitlementsResource) -> None:
        self._entitlements = entitlements

        self.create = async_to_raw_response_wrapper(
            entitlements.create,
        )
        self.update = async_to_raw_response_wrapper(
            entitlements.update,
        )
        self.list = async_to_raw_response_wrapper(
            entitlements.list,
        )
        self.delete = async_to_raw_response_wrapper(
            entitlements.delete,
        )


class EntitlementsResourceWithStreamingResponse:
    def __init__(self, entitlements: EntitlementsResource) -> None:
        self._entitlements = entitlements

        self.create = to_streamed_response_wrapper(
            entitlements.create,
        )
        self.update = to_streamed_response_wrapper(
            entitlements.update,
        )
        self.list = to_streamed_response_wrapper(
            entitlements.list,
        )
        self.delete = to_streamed_response_wrapper(
            entitlements.delete,
        )


class AsyncEntitlementsResourceWithStreamingResponse:
    def __init__(self, entitlements: AsyncEntitlementsResource) -> None:
        self._entitlements = entitlements

        self.create = async_to_streamed_response_wrapper(
            entitlements.create,
        )
        self.update = async_to_streamed_response_wrapper(
            entitlements.update,
        )
        self.list = async_to_streamed_response_wrapper(
            entitlements.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            entitlements.delete,
        )
